<?php

namespace Teknisa\MAN\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\MAN\Utilities;

class RelDRFService {

	public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    public function buildReportParameters($conditions){
        $additionalParameters = self::findAdditionalParams($conditions); 
        $dateParams           = self::buildUtilDateParams($conditions);
        $cdtipocusto          = self::findTipoCusto($conditions);
        $params = array(
            'CDFILIAL'         => $conditions['CDFILIAL'],
            'DATAINI'          => $conditions['DTINICROP'],
            'DATAFIN'          => $conditions['DTFINALROP'],
            'NRDIASUTEIS'      => $dateParams['DIASUTEIS'],
            'NRDIASCORRIDOS'   => $dateParams['DIASCORRIDOS'],
            'CDGRUPPROD'       => $additionalParameters['CDGRUPPROD'],
            'IDFILAGRROP'      => $additionalParameters['IDFILIALAGRROP'],
            'NRMETROFILIAL'    => $additionalParameters['NRMETROFILIAL'],
            'NRTIPOCUSTO'      => $cdtipocusto,
            'DTLANCROP'        => $conditions['DTINICROP']
        );
        $params = array(
            'ParametersRow' => $params,
            'ReportName'    => 'demonstrativoResultForsale'
        );
        return $params;
    }
    
    private function findAdditionalParams($conditions){
        $params = array (
            ':CDFILIAL' => $conditions['CDFILIAL']
        );
        $additionalParams = $this->entityManager->getConnection()->fetchAll(self::FETCH_PARAMS, $params);
        if(is_null($additionalParams[0]['NRMETROFILIAL'])){
            $additionalParams[0]['NRMETROFILIAL'] = 1;
        }
        if(is_null($additionalParams[0]['CDGRUPPROD'])){
            $additionalParams[0]['CDGRUPPROD'] = '_';
        }
        return $additionalParams[0];
    }
    
    private function buildUtilDateParams($conditions){
        $formatDates = self::convertDateToAmericanFormat($conditions);
        $dataIni     = \DateTime::createFromFormat('Y-m-d', $formatDates['DATAINI']);
        $dataFin     = \DateTime::createFromFormat('Y-m-d', $formatDates['DATAFIN']);
        $nrDiasCorridos = (date_diff($dataIni, $dataFin)->days) + 1;
        $params = array(
            ':CDFILIAL' => $conditions['CDFILIAL'],
            ':DATAINI'  => $conditions['DTINICROP'],
            ':DATAFIN'  => $conditions['DTFINALROP']
        );
        $feriados = $this->entityManager->getConnection()->fetchAll(self::FETCH_HOLIDAYS, $params);
        if(empty($feriados)){
            $nrDiasUteis = $nrDiasCorridos;
        } else {
            $nrDiasUteis = $nrDiasCorridos - $feriados[0]['TOTAL'];
        }
        $dateParams = array(
            'DIASUTEIS'    => $nrDiasUteis,
            'DIASCORRIDOS' => $nrDiasCorridos
        );
        return $dateParams;
    }
    
    private function convertDateToAmericanFormat($conditions){
        $iniArray = explode('/', $conditions['DTINICROP']);
        $finArray = explode('/', $conditions['DTFINALROP']);
        $formatDates = array(
            'DATAINI' => $iniArray[2].'-'.$iniArray[1].'-'.$iniArray[0],
            'DATAFIN' => $finalOnFormat = $finArray[2].'-'.$finArray[1].'-'.$finArray[0]
        );
        return $formatDates;
    }
    
    private function findTipoCusto($conditions){
        $params = array (
            ':CDFILIAL'  =>     $conditions['CDFILIAL']
        );
        $cdtipocusto = $this->entityManager->getConnection()->fetchAll(self::FETCH_CDTIPOCUSTO, $params);
        return $cdtipocusto[0]['CDTIPOCUSTO'];
    }
    
    
    const FETCH_PARAMS = "
       SELECT P.CDFILIAL, IDUTPADRROP,   QTFUNCFILIROP, VRADIANTROP, IDSERVRETROP, CDGRUPPRODDRL AS CDGRUPPROD , IDFATUROP, IDEFETIVOROP,
          IDCOMPRAROP, IDNFSAIDAROP,  IDINVINICROP,   IDIMPLANTROP,  IDINVFINAROP, IDAJUSTEROP, IDRETIRADAROP,  IDDESPDIVROP,  IDDESPMOBROP,
          IDDESPENTROP, IDUTPADRROPFIL, IDFILIALAGRROP, IDDETALHEFATUR, IDDETALEFET, NRMETROFILIAL 
        FROM PARFILIALROP P, PARAMGERAL
        WHERE P.CDFILIAL  = TIPOPARAM_DRO(:CDFILIAL)
    ";
    
    const FETCH_HOLIDAYS = "
        SELECT DTFERIFILI, COUNT(*) AS TOTAL
        FROM FERIFILI
        WHERE CDFILIAL   = :CDFILIAL
          AND DTFERIFILI BETWEEN :DATAINI AND :DATAFIN 
        GROUP BY DTFERIFILI
    ";
    
    const FETCH_CDTIPOCUSTO = "
        SELECT CDTIPOFATUR, IDFDIAENTREST,IDFDIASAIDEST,IDFDIARETIEST, IDFDIACARDAP,
          IDFDIAREQEST, IDFDIAREQPLA, IDFDIAEFEFAT, IDFDIAEFEPLA, NVL(CDTIPCUSTEST, '03') CDTIPOCUSTO
        FROM PARAMFILIAL
        WHERE CDFILIAL = :CDFILIAL
    ";
}